@@ -6,6 +6,7 @@ __all__ = [
     'address_no_brackets',
     'change_usergroup',
     'change_uidgid',
+    'check_ssl_key_and_cert',
     'deliver_maildir',
     'eval_bool',
     'expand_user_vars',
@@ -49,23 +50,31 @@ _bool_values = {
 }
 
 #######################################
-def lock_file(file):
-    '''Do fcntl file locking.'''
-    fcntl.flock(file.fileno(), fcntl.LOCK_EX)
+def lock_file(file, locktype):
+    '''Do file locking.'''
+    assert locktype in ('lockf', 'flock'), 'unknown lock type %s' % locktype
+    if locktype == 'lockf':
+        fcntl.lockf(file, fcntl.LOCK_EX)
+    elif locktype == 'flock':
+        fcntl.flock(file, fcntl.LOCK_EX)
 
 #######################################
-def unlock_file(file):
-    '''Do fcntl file unlocking.'''
-    fcntl.flock(file.fileno(), fcntl.LOCK_UN)
+def unlock_file(file, locktype):
+    '''Do file unlocking.'''
+    assert locktype in ('lockf', 'flock'), 'unknown lock type %s' % locktype
+    if locktype == 'lockf':
+        fcntl.lockf(file, fcntl.LOCK_UN)
+    elif locktype == 'flock':
+        fcntl.flock(file, fcntl.LOCK_UN)
 
 #######################################
-def safe_open(path, mode):
+def safe_open(path, mode, permissions=0600):
     '''Open a file path safely.
     '''
     if os.name != 'posix':
         return open(path, mode)
     try:
-        fd = os.open(path, os.O_RDWR | os.O_CREAT | os.O_EXCL, 0600)
+        fd = os.open(path, os.O_RDWR | os.O_CREAT | os.O_EXCL, permissions)
         file = os.fdopen(fd, mode)
     except OSError, o:
         raise getmailDeliveryError('failure opening %s (%s)' % (path, o))
@@ -84,6 +93,13 @@ class updatefile(object):
         self.closed = False
         self.filename = filename
         self.tmpname = filename + '.tmp.%d' % os.getpid()
+        # If the target is a symlink, the rename-on-close semantics of this
+        # class would break the symlink, replacing it with the new file.
+        # Instead, follow the symlink here, and replace the target file on
+        # close.
+        while os.path.islink(filename):
+            filename = os.path.join(os.path.dirname(filename),
+                                    os.readlink(filename))
         try:
             f = safe_open(self.tmpname, 'wb')
         except IOError, (code, msg):
@@ -107,6 +123,7 @@ class updatefile(object):
         if self.closed or not hasattr(self, 'file'):
             return
         self.file.flush()
+        os.fsync(self.file.fileno())
         self.file.close()
         os.rename(self.tmpname, self.filename)
         self.closed = True
@@ -138,14 +155,14 @@ class logfile(object):
 
     def write(self, s):
         try:
-            lock_file(self.file)
+            lock_file(self.file, 'flock')
             # Seek to end
             self.file.seek(0, 2)
             self.file.write(time.strftime(logtimeformat, time.localtime())
-                + ' ' + s.rstrip() + os.linesep)
+                            + ' ' + s.rstrip() + os.linesep)
             self.file.flush()
         finally:
-            unlock_file(self.file)
+            unlock_file(self.file, 'flock')
 
 #######################################
 def format_params(d, maskitems=('password', ), skipitems=()):
@@ -179,24 +196,30 @@ def is_maildir(d):
     '''
     dir_parent = os.path.dirname(d.endswith('/') and d[:-1] or d)
     if not os.access(dir_parent, os.X_OK):
-        raise getmailConfigurationError('cannot read contents of parent '
-            'directory of %s - check permissions and ownership' % d)
+        raise getmailConfigurationError(
+            'cannot read contents of parent directory of %s '
+            '- check permissions and ownership' % d
+        )
     if not os.path.isdir(d):
         return False
     if not os.access(d, os.X_OK):
-        raise getmailConfigurationError('cannot read contents of '
-            'directory %s - check permissions and ownership' % d)
+        raise getmailConfigurationError(
+            'cannot read contents of directory %s '
+            '- check permissions and ownership' % d
+        )
     for sub in ('tmp', 'cur', 'new'):
         subdir = os.path.join(d, sub)
         if not os.path.isdir(subdir):
             return False
         if not os.access(subdir, os.W_OK):
-            raise getmailConfigurationError('cannot write to maildir '
-                '%s - check permissions and ownership' % d)
+            raise getmailConfigurationError(
+                'cannot write to maildir %s '
+                '- check permissions and ownership' % d
+            )
     return True
 
 #######################################
-def deliver_maildir(maildirpath, data, hostname, dcount=None):
+def deliver_maildir(maildirpath, data, hostname, dcount=None, filemode=0600):
     '''Reliably deliver a mail message into a Maildir.  Uses Dan Bernstein's
     documented rules for maildir delivery, and the updated naming convention
     for new files (modern delivery identifiers).  See
@@ -227,8 +250,10 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None):
         if info['deliverycount'] is not None:
             info['unique'] += 'Q%(deliverycount)s' % info
         try:
-            info['unique'] += 'R%s' % ''.join(['%02x' % ord(char)
-                for char in open('/dev/urandom', 'rb').read(8)])
+            info['unique'] += 'R%s' % ''.join(
+                ['%02x' % ord(char)
+                 for char in open('/dev/urandom', 'rb').read(8)]
+            )
         except StandardError:
             pass
 
@@ -263,7 +288,7 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None):
 
     # Open file to write
     try:
-        f = safe_open(fname_tmp, 'wb')
+        f = safe_open(fname_tmp, 'wb', filemode)
         f.write(data)
         f.flush()
         os.fsync(f.fileno())
@@ -272,7 +297,7 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None):
     except IOError, o:
         signal.alarm(0)
         raise getmailDeliveryError('failure writing file %s (%s)'
-            % (fname_tmp, o))
+                                   % (fname_tmp, o))
 
     # Move message file from Maildir/tmp to Maildir/new
     try:
@@ -288,7 +313,7 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None):
         except StandardError:
             pass
         raise getmailDeliveryError('failure renaming "%s" to "%s"'
-            % (fname_tmp, fname_new))
+                                   % (fname_tmp, fname_new))
 
     # Delivery done
 
@@ -318,8 +343,10 @@ def eval_bool(s):
     try:
         return _bool_values[str(s).lower()]
     except KeyError:
-        raise getmailConfigurationError('boolean parameter requires value'
-            ' to be one of true or false, not "%s"' % s)
+        raise getmailConfigurationError(
+            'boolean parameter requires value to be one of true or false, '
+            'not "%s"' % s
+        )
 
 #######################################
 def gid_of_uid(uid):
@@ -376,7 +403,7 @@ def change_uidgid(logger=None, uid=None, gid=None):
                 os.setreuid(uid, uid)
     except OSError, o:
         raise getmailDeliveryError('change UID/GID to %s/%s failed (%s)'
-            % (uid, gid, o))
+                                   % (uid, gid, o))
 
 #######################################
 def format_header(name, line):
@@ -384,7 +411,7 @@ def format_header(name, line):
     '''
     header = ''
     line = (name.strip() + ': '
-        + ' '.join([part.strip() for part in line.splitlines()]))
+            + ' '.join([part.strip() for part in line.splitlines()]))
     # Split into lines of maximum 78 characters long plus newline, if
     # possible.  A long line may result if no space characters are present.
     while line and len(line) > 78:
@@ -420,3 +447,25 @@ def localhostname():
     if '.' in n:
         return n
     return socket.getfqdn()
+
+#######################################
+def check_ssl_key_and_cert(conf):
+    keyfile = conf['keyfile']
+    if keyfile is not None:
+        keyfile = expand_user_vars(keyfile)
+    certfile = conf['certfile']
+    if certfile is not None:
+        certfile = expand_user_vars(certfile)
+    if keyfile and not os.path.isfile(keyfile):
+        raise getmailConfigurationError(
+            'optional keyfile must be path to a valid file'
+        )
+    if certfile and not os.path.isfile(certfile):
+        raise getmailConfigurationError(
+            'optional certfile must be path to a valid file'
+        )
+    if (keyfile is None) ^ (certfile is None):
+        raise getmailConfigurationError(
+            'optional certfile and keyfile must be supplied together'
+        )
+    return (keyfile, certfile)
